/************************************************************************************************

 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : November 2007

 File          : SPI_Master.c

 Hardware      : ADuC706x

 Description   : This SPI Master example acts as interface between the PC based application SPISerial.exe
 				and the Slave EPROM. 
				This application accepts commands via the serial port from the PC, converts these to SPI
				and passes them to the Slave/EPROM device.
				 Using the command window based executable, SPISerial.exe, data bytes
				are sent serially to the Master via UART. Depending on whether	the application sent
				a write or read command, the Master sends the appropriate command to the Slave via 
				SPI channel. If a slave read is requested, the Master will request the data
				from the SPIserial.exe specified address and return the data to the PC.
				

*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT17 0x20000
#define BIT25 0x2000000

#include<aduc7060.h>

// Function Definitions
void Uart_Packet_Handler( volatile unsigned int value, volatile unsigned char Byte_Number);
unsigned char SPIWritePacket( unsigned int length);
unsigned char SPIReadPacket( unsigned int length);
void delay(int);

// Application specific variables
volatile unsigned int rxDat[256];	  // 256 byte buffer-array for SPI receive data
volatile unsigned char ucRxCount = 0; // rxDat array control variable.
volatile unsigned char szTxData[256];	 // Buffer used during SPI "writes to Slave"
volatile unsigned char ucTxCount = 0; // rxDat array control variable.
volatile unsigned char ucComRx = 0;   // Variable used to read UART Rx buffer contents into 
volatile unsigned char ucMsgType = 0; // Indicates SPISerial.exe application requested SPI read or write
volatile unsigned int ucLength = 0;  // Indicates SPISerial.exe "number of bytes" for SPI read or write
volatile unsigned int ucStartAddress = 0;	// Indicates SPISerial.exe "Start address" for SPI read or write
volatile unsigned char ucSendSPIPacket = 0;	// Used in main loop to indicate SPI message is ready for transmission
	
volatile unsigned int ucRdCnt = 0;	 // Counter for UART receive messages
volatile unsigned char ucDataCnt = 0; // Message Parser counter for UART messages
volatile unsigned char ucCOMSTA0 = 0; // Variable used to store COMSTA0, UART status register
volatile unsigned char ucCOMIID0 = 0; // Variable used to store COMIID0, UART Interrupt status register	 
unsigned int uiIRQSTA = 0;			  // Variable used to store IRQSTA, UART interrupt status register
volatile unsigned char ucTxBufferEmpty = 0;	// Indicates UART Tx buffer empty
volatile unsigned char ucTransmitInProgress =0; // Indicates SPI Tx FIFO ready for data
volatile unsigned char ucReceiveInProgress = 0; // Indicates SPI Rx sequence with the SLave is complete
volatile unsigned char ucSPIWrCnt = 0; // Counter for controlling SPI writes
volatile unsigned char ucSPICNT = 0;	   // The 4 following variables handle SPI reads of greater than 8 bytes
volatile unsigned char ucSPICNTReload = 0;
volatile unsigned char ucNumReloads =0;	
volatile unsigned char ucRemainder =0;	
volatile unsigned char ucValidUARTPacket = 0;  // Indicates a valid UART packet has been received.
volatile unsigned char ucRdLength = 0;	   // Stores number of bytes to be read from Slave via SPI
volatile unsigned char ucBytesToRD = 0;			// Stores number of bytes to be read from Slave via SPI
volatile unsigned char ucToggleCnt = 0;
volatile unsigned char ucStringCnt = 0;
volatile unsigned char ucLoadVal = 0;
volatile unsigned char ucLoadCnt = 0;
volatile unsigned char StartByteGone = 0;
unsigned char ucWaitingForXIRQ0 = 1; // Flag to begin reading from Slave - controlled by XIRQ0
int main(void)
{
    POWKEY1 = 0x1;
	POWCON0 = 0x78;		   // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;

	POWKEY3 = 0x76;
	POWCON0 = 0x124;	   // Set the Peripheral clocks to max speed of 10.24Mhz
	POWKEY4 = 0xB1;

	GP1CON = BIT0 + BIT4;  // Select UART functionality for P1.0/P1.1
		// Initialize the UART for 9600-8-N
	COMCON0 = BIT7;			// Enable access to COMDIV registers
	COMDIV0 = 0x21;
	COMDIV1 = 0x00;
	//COMDIV2 = 0x21 + BIT11; // Enable fractional divider for more accurate baud rate setting
	COMCON0 = BIT0 + BIT1 + BIT2;
	COMIEN0 = BIT0 + BIT1;	 // Enable UART interrupts when Rx full and Tx buffer empty.

	// Configure SPI port
	// Configure P0.0, P0.1,P0.2 and P0.3 for SPI mode
	GP0CON0 = BIT0 + BIT4 + BIT8 + BIT12; // Select SPI/I2C alternative function for P0[0...3]
	GP0KEY1 = 0x7;		   //Write to GP0KEY1
	GP0CON1	= 0x00;	   // Select SPI functionality for P0.0 to  P0.3
	GP0KEY2 = 0x13;		   //Write to GP0KEY2

	
	SPICON = BIT0 + BIT1   // Enable SPI + Enable SPI Master
		    + BIT6  // Clk pulses high at start of each bit + Initiate transfer on write to Tx FIFO
		   + BIT11 ;		   //Continuous transfer mode + Enable IRQ on 2 byte transfer
	SPIDIV  = 0x20;	   		// Select 101kHz clock rate
	IRQCONE = BIT1;			// Rising edge of XIRQ0
	IRQEN = BIT12 + BIT13 + BIT11; 		// Enable SPI + XIRQ0 + UART interrupt

	ucTxCount = 0;	
	ucWaitingForXIRQ0 = 1;
	ucRxCount = 0;
	while (1)
	{
	
		if (ucSendSPIPacket == 1)
		{
			if (ucMsgType == 0)
			{
			 	SPIWritePacket(ucLength);
				ucSendSPIPacket = 0;
				ucDataCnt = 0;
				ucRdCnt = 0;
			}
			else
			{
				SPIReadPacket(ucLength);
				ucSendSPIPacket = 0;
				ucDataCnt = 0;
				ucRdCnt = 0;
				ucLoadCnt = 0;
			}
		}	
	}
 	return 0;
}
// This Function Parses the UART packets received from the PC application.
void Uart_Packet_Handler( volatile unsigned int value, volatile unsigned char Byte_Number)
{
  switch(Byte_Number)
  {
	   case 1:	//Read/Write indicator
	   	{
			ucMsgType = value;
			break;
		} 
	   case 2:  //	Length of data in packet
	   {
	   		ucLength = value;
			break;
	   }	   
	   case 6 : // Memory Start Address
	   {
	   		ucStartAddress = value;
			break;
	   }
	   case 9 : // Memory Start Address
	   {
	   		if (ucMsgType == 1)	// if msg type is a read - reset buffer
			{
				ucDataCnt = 0;	// Reset message parse register
				ucSendSPIPacket = 1;  //Send Packet to the slave
				ucRdCnt = 0;
				ucValidUARTPacket = 0;
			}
			break;
	   }
		default :
	   break;
	}
}
// Fucntion used to commence SPI Read sequence
unsigned char SPIReadPacket( unsigned int length)
{
	unsigned char n = 0;
	volatile unsigned int test1 = 0;
	volatile unsigned int test2 = 0;

	
	ucBytesToRD = length; 	 
	// Transmit	EEPROM address to read from
	ucTransmitInProgress = 1;
	StartByteGone = 1;

	ucLength = 0;
	ucRxCount = 0;
	IRQCLR = BIT12;
	SPITX =   ucMsgType;		// Message type is a read
	while ((SPISTA  & 0xE) != 0x0)
	{
	test1 = SPISTA;
	}
	SPITX =   ucStartAddress;		// send EEPROM Address to start at
	while ((SPISTA  & 0xE) != 0x0)
	{
	test2 = SPISTA;
	}
	SPICON &= ~BIT6; // Generate Interrupt on receiving data
	SPICON |= BIT12;			// Flush Master Rx FIFO
	SPICON &= ~BIT12;
	IRQEN |= BIT12;
	rxDat[ucRxCount] = SPIRX;
   		
	ucReceiveInProgress = 1;

	while (ucReceiveInProgress == 1) {}
	SPICON |= BIT6; // Generate Interrupt on Transmitting data 
	delay (0x12000);
	SPICON |= BIT13;			// Flush Master Tx FIFO
	SPICON &= ~BIT13;
	SPICON |= BIT12;			// Flush Master Rx FIFO
	SPICON &= ~BIT12;
	ucRdLength = 0;
	ucDataCnt = 0;
//	ucBytesToRD = 0;
	ucRdCnt = 0;
	ucTxBufferEmpty = 0;
	COMTX = rxDat[0];
	ucSendSPIPacket = 0;
	for (n = 1; n < ucBytesToRD +1 ; n++	)
	{
		while (ucTxBufferEmpty != 1){}
		ucTxBufferEmpty = 0;
		COMTX = rxDat[n];
	}
	return 1;
}
// This function Starts an SPI Write sequence
unsigned char SPIWritePacket(unsigned int length )
{
	// Transmit
	ucTransmitInProgress = 1;
	
	IRQCLR = BIT12;
	SPICON |= BIT13;			// Flush Master Tx FIFO
	SPICON &= ~BIT13;
	SPICON |= BIT12;			// Flush Master Rx FIFO
	SPICON &= ~BIT12;
	SPITX =   ucMsgType;		// Message type is a Write
	while ((SPISTA  & 0xE) != 0x0)
	{

	}
	IRQEN |= BIT12;
	SPITX =   ucStartAddress;		// send EEPROM Address to start at
//	while ((SPISTA  & 0xE) != 0x0)
//	{	}
	

	while (ucTransmitInProgress == 1)
	{}
	
	return 1;
}
// Simple Delay routine
void delay (int length)
{
	while (length >0)
    	length--;
}
void IRQ_Handler(void) __irq
{
	unsigned long IRQSTATUS = 0;
	unsigned int SPIMSTATUS = 0;
	unsigned char Underflow = 0;

	IRQSTATUS = IRQSTA;	   // Read off IRQSTA register
	if ((IRQSTATUS & BIT13) == BIT13)	//XIRQ0 interrupt source
	{
		ucWaitingForXIRQ0 = 0;			// Enable Read request from Slave.
		IRQCLRE = BIT13;
	}
		IRQSTATUS = IRQSTA;	   // Read off IRQSTA register

	// UART ISR
	if ((IRQSTATUS & BIT11) == BIT11)	//UART interrupt source
	{
		ucCOMSTA0 = COMSTA0;
		ucCOMIID0 = COMIID0;
		if ((ucCOMIID0 & 0x2) == 0x2)	  // Transmit buffer empty
		{
		  ucTxBufferEmpty = 1;
		}
		
		if ((ucCOMIID0 & 0x4) == 0x4)	  // Receive byte
		{
			ucComRx	= COMRX;
			if 	((ucRdCnt == 0) && (ucComRx == 8))	 // Received Start character 0x08
				ucValidUARTPacket = 1;
			if ((ucRdCnt < 10) && (ucSendSPIPacket == 0) && (ucValidUARTPacket == 1))
			{					
				Uart_Packet_Handler( ucComRx, ucRdCnt);
				ucRdCnt++;
			}
			else if ((ucSendSPIPacket == 0) && (ucValidUARTPacket == 1))
			{
				szTxData[ucDataCnt++] = COMRX;				 
				if (ucDataCnt == ucLength)
				{
			  		ucDataCnt = 0;	// Reset message parse register
					ucSendSPIPacket = 1;  //Send Packet to the slave
					ucValidUARTPacket = 0;
				}
			}
		}
	}
	if ((IRQSTATUS & BIT12) == BIT12)	//If SPI Master interrupt source
	{
	   SPIMSTATUS = SPISTA;
	   if ((SPIMSTATUS & BIT4) == BIT4) // SPI Tx underflow
	   {
			if (ucTxCount < ucLength)		   // Have max 6 bytes been sent?
				SPITX = szTxData[ucTxCount++];	 // Load Tx buffer
				
			else 
			{
				ucTransmitInProgress = 0;
				ucTxCount = 0; 
			
			}
	   }
	   if ((SPIMSTATUS & BIT5) == BIT5) // If SPI Master Tx IRQ
	   {
			if (ucTxCount < ucLength)		   // Have max 6 bytes been sent?
				SPITX = szTxData[ucTxCount++];	 // Load Tx buffer
				
			else 
			{
				ucTransmitInProgress = 0;
				ucTxCount = 0; 
			
			}
	   }
	   if ((SPIMSTATUS & BIT6) == BIT6) // If SPI Master Rx IRQ
	   {
			if (ucRxCount < ucBytesToRD)		   // Have max 6 bytes been received?
			{
				rxDat[ucRxCount] = SPIRX;  // Read Rx buffer
				ucRxCount++;
			}
			else
			{
			 	ucReceiveInProgress = 0;
			}
	   }
	}
}


